<?php

namespace Drupal\mail_box_management\Plugin;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\node\Entity\Node;

class EntitySaver {

  private array $entities = [];

  /**
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function add(array $entities, $owner_uid = 1): void {

    if(array_key_exists(0, $entities)) {
      foreach ($entities as $key => $entity) {
        if(!empty($entity['header']['Subject'])) {

          $attachments = [];
          if(!empty($entity['attachments'])) {
            foreach($entity['attachments'] as $attachment) {
              $file_type = strtolower($attachment['type']);
              $stored_path = $this->write($file_type, base64_decode($attachment['content']));
              $file = File::create([
                'uri' => $stored_path,
              ]);
              $file->setOwnerId($owner_uid);
              $file->setPermanent();
              $file->save();
              $attachment[] = [ 'target_id'=> $file->id() ];
            }
          }

          $this->entities[] = [
            'title' => $entity['header']['Subject'],
            'from_email' => $entity['header']['from_email'] ?? '',
            'from_name' => $entity['header']['from_name'] ?? '',
            'to_email' => $entity['header']['to_email'] ?? '',
            'to_name' => $entity['header']['to_name'] ?? '',
            'content' => $entity['body']['content'] ?? '',
            'attachment' => $attachments,
            'type' => 'mail_box_content'
          ];
        }
      }
    }
    else {
      if(!empty($entities['header']['Subject'])) {
        $entity = $entities;
        $attachments = [];
        if(!empty($entity['attachments'])) {
          foreach($entity['attachments'] as $key=>$attachment) {
            $file_type = strtolower($attachment['type']);
            $stored_path = $this->write($file_type, base64_decode($attachment['content']));
            $file = File::create([
              'uri' => $stored_path,
            ]);
            $file->setOwnerId($owner_uid);
            $file->setPermanent();
            $file->save();
            $attachments[] = [ 'target_id'=> $file->id() ];
          }
        }

        $this->entities[] = [
          'title' => $entity['header']['Subject'],
          'from_email' => $entity['header']['from_email'] ?? '',
          'from_name' => $entity['header']['from_name'] ?? '',
          'to_email' => $entity['header']['to_email'] ?? '',
          'to_name' => $entity['header']['to_name'] ?? '',
          'content' => $entity['body']['content'] ?? '',
          'attachment' => $attachments,
          'type' => 'mail_box_content'
        ];
      }
    }
  }

  public function save(): bool {

    $flags = [];
    foreach($this->entities as $entity) {
      $flags[] = !empty(Node::create($entity)->save());
    }
    return in_array(TRUE, $flags);
  }

  private function write(string $file_type, string $data): ?string {
    $directory = "public://mailbox_management/attachments";
    $file_system = mail_box_management_service('file_system');
    try {
      if ($file_system->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY)) {
        $storage_path = $directory . '/attachment-' . time() . '.'.$file_type;
        $file_system->saveData($data, $storage_path);
        return $storage_path;
      }
    }
    catch (\Exception) {
    }
    return null;
  }

  public static function create(array $data, int $owner_id): EntitySaver {
    $new = new static();
    $new->add($data, $owner_id);
    return $new;
  }
}
